{# -*- coding: utf-8 -*-

    This file is part of Invenio.
    Copyright (C) 2016-2020 CERN.

    Invenio is free software; you can redistribute it and/or modify it
    under the terms of the MIT License; see LICENSE file for more details.
#}

{% extends "invenio_communities/base.html" %}

{%- set title = _("Search communities") -%}

{%- block javascript %}
    {{ super() }}
    {{ webpack['invenio-communities-search.js'] }}
{%- endblock %}

{%- block page_body %}
<div class="ui container fluid page-subheader-outer compact ml-0-mobile mr-0-mobile">
  <div class="ui container community-search page-subheader">
    <h3 class="ui header">{{_("Communities")}}</h3>
  </div>
</div>
<div id="communities-search" class="rel-mt-3"
    data-invenio-search-config='{{search_app_communities_config(app_id="InvenioCommunities.Search") | tojson }}'
    data-permissions='{{ permissions | tojson }}'
    >
</div>
{%- endblock page_body %}
